float4 BaseColor;
float Diffuse;
float4 Ambient;
float4 Specular;
float SpecularPower;
float Emissive;
float3 LightDir;
float3 ViewDir;
texture ColorMap;
texture AlphaMap;

sampler ColorMapSampler = sampler_state
{
	Texture = <ColorMap>;
};

sampler AlphaMapSampler = sampler_state
{
	Texture = <AlphaMap>;
};

struct VS_OUTPUT
{
	float4 Pos : POSITION;
	float4 Color : COLOR0;
	float3 Normal : TEXCOORD0;
	float2 TexCoord : TEXCOORD1;
};

/// Shader to render TR4 rooms like the tomb4.exe engine
/// In the material settings, set shader to "Blinn" and check "Vertex color".
float4 PS(VS_OUTPUT In) : COLOR0
{
	float4 tex_col = tex2D(ColorMapSampler, In.TexCoord);
	float4 alpha_col = tex2D(AlphaMapSampler, In.TexCoord);
	float3 Norm = normalize(In.Normal);
	float dif_coef = dot(Norm, LightDir);
	
	float3 Reflect = normalize(2 * dif_coef * Norm - LightDir);
	float spc_coef = pow(saturate(dot(Reflect, ViewDir)), SpecularPower);

	float4 col;
	In.Color.a = 1; /// Remove any vertex transparency for simplicity. 
					/// The game engine does recognize this type of transparency but
					/// Metasequoia would have to detect whether to apply it from the 
					/// uv coordinates.
	
	/// tomb4.exe shades rooms as follows :
	if ((In.Color.r + In.Color.g + In.Color.b) > 1.5)
		col = tex_col + float4(
			0.5 * (In.Color.r - 0.5),
			0.5 * (In.Color.g - 0.5),
			0.5 * (In.Color.b - 0.5),
			0);
	else 
		col = tex_col * float4(
			2 * In.Color.r,
			2 * In.Color.g,
			2 * In.Color.b,
			1);
			
	return saturate(col);
}

